@@ -25,15 +25,13 @@
 #endif
 
 #include "gstgtkwaylandsink.h"
+
+#include <gdk/gdk.h>
+#include <gst/wayland/wayland.h>
+
 #include "gstgtkutils.h"
 #include "gtkgstwaylandwidget.h"
 
-#include <drm_fourcc.h>
-#include <gdk/gdk.h>
-#include <gst/allocators/allocators.h>
-#include <gst/video/gstvideodmabufpool.h>
-#include <gst/wayland/wayland.h>
-
 #ifdef GDK_WINDOWING_WAYLAND
 #include <gdk/gdkwayland.h>
 #else
@@ -43,11 +41,21 @@
 #define GST_CAT_DEFAULT gst_debug_gtk_wayland_sink
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
+#ifndef GST_CAPS_FEATURE_MEMORY_DMABUF
+#define GST_CAPS_FEATURE_MEMORY_DMABUF "memory:DMABuf"
+#endif
+
+#define WL_VIDEO_FORMATS \
+  "{ BGRx, BGRA, RGBx, xBGR, xRGB, RGBA, ABGR, ARGB, RGB, BGR, " \
+  "RGB16, BGR16, YUY2, YVYU, UYVY, AYUV, NV12, NV21, NV16, NV61, " \
+  "YUV9, YVU9, Y41B, I420, YV12, Y42B, v308 }"
+
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_DMA_DRM_CAPS_MAKE ";"
-        GST_VIDEO_CAPS_MAKE (GST_WL_VIDEO_FORMATS))
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE (WL_VIDEO_FORMATS) ";"
+        GST_VIDEO_CAPS_MAKE_WITH_FEATURES (GST_CAPS_FEATURE_MEMORY_DMABUF,
+            WL_VIDEO_FORMATS))
     );
 
 static void gst_gtk_wayland_sink_get_property (GObject * object,
@@ -83,8 +91,7 @@ enum
   PROP_0,
   PROP_WIDGET,
   PROP_DISPLAY,
-  PROP_ROTATE_METHOD,
-  PROP_DRM_DEVICE,
+  PROP_ROTATE_METHOD
 };
 
 typedef struct _GstGtkWaylandSinkPrivate
@@ -102,25 +109,19 @@ typedef struct _GstGtkWaylandSinkPrivate
 
   GstBufferPool *pool;
   GstBuffer *last_buffer;
+  gboolean use_dmabuf;
 
-  gboolean render_info_changed;
-  GstVideoInfo render_info;
+  gboolean video_info_changed;
   GstVideoInfo video_info;
-  GstVideoInfoDmaDrm drm_info;
-  GstVideoMasteringDisplayInfo minfo;
-  GstVideoContentLightLevel linfo;
-  gboolean have_mastering_info;
-  gboolean have_light_info;
-  GstCaps *caps;
 
+  gboolean redraw_pending;
   GMutex render_lock;
 
   GstVideoOrientationMethod sink_rotate_method;
   GstVideoOrientationMethod tag_rotate_method;
   GstVideoOrientationMethod current_rotate_method;
 
-  gchar *drm_device;
-  gboolean skip_dumb_buffer_copy;
+  struct wl_callback *callback;
 } GstGtkWaylandSinkPrivate;
 
 #define gst_gtk_wayland_sink_parent_class parent_class
@@ -161,25 +162,13 @@ gst_gtk_wayland_sink_class_init (GstGtkWaylandSinkClass * klass)
           "rotate method",
           "rotate method",
           GST_TYPE_VIDEO_ORIENTATION_METHOD, GST_VIDEO_ORIENTATION_IDENTITY,
-          G_PARAM_READWRITE | GST_PARAM_MUTABLE_PLAYING |
-          G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GstGtkWaylandSink:drm-device:
-   *
-   * Since: 1.24
-   */
-  g_object_class_install_property (gobject_class, PROP_DRM_DEVICE,
-      g_param_spec_string ("drm-device", "DRM Device", "Path of the "
-          "DRM device to use for dumb buffer allocation",
-          NULL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_gtk_wayland_sink_change_state);
 
-  gst_element_class_set_static_metadata (gstelement_class,
-      "Gtk Wayland Video Sink", "Sink/Video",
+  gst_element_class_set_metadata (gstelement_class, "Gtk Wayland Video Sink",
+      "Sink/Video",
       "A video sink that renders to a GtkWidget using Wayland API",
       "George Kiagiadakis <george.kiagiadakis@collabora.com>");
 
@@ -214,14 +203,7 @@ gst_gtk_wayland_sink_finalize (GObject * object)
   GstGtkWaylandSinkPrivate *priv =
       gst_gtk_wayland_sink_get_instance_private (self);
 
-  g_clear_object (&priv->display);
-  g_clear_object (&priv->wl_window);
-  g_clear_object (&priv->pool);
-
   g_clear_object (&priv->gtk_widget);
-  gst_clear_caps (&priv->caps);
-
-  g_free (priv->drm_device);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -244,7 +226,6 @@ window_destroy_cb (GtkWidget * widget, GstGtkWaylandSink * self)
       gst_gtk_wayland_sink_get_instance_private (self);
 
   GST_OBJECT_LOCK (self);
-  g_clear_object (&priv->wl_window);
   priv->gtk_window = NULL;
   GST_OBJECT_UNLOCK (self);
 
@@ -396,11 +377,6 @@ gst_gtk_wayland_sink_get_property (GObject * object, guint prop_id,
     case PROP_ROTATE_METHOD:
       g_value_set_enum (value, priv->current_rotate_method);
       break;
-    case PROP_DRM_DEVICE:
-      GST_OBJECT_LOCK (self);
-      g_value_set_string (value, priv->drm_device);
-      GST_OBJECT_UNLOCK (self);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -412,20 +388,12 @@ gst_gtk_wayland_sink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
   GstGtkWaylandSink *self = GST_GTK_WAYLAND_SINK (object);
-  GstGtkWaylandSinkPrivate *priv =
-      gst_gtk_wayland_sink_get_instance_private (self);
 
   switch (prop_id) {
     case PROP_ROTATE_METHOD:
       gst_gtk_wayland_sink_set_rotate_method (self, g_value_get_enum (value),
           FALSE);
       break;
-    case PROP_DRM_DEVICE:
-      /* G_PARAM_CONSTRUCT_ONLY */
-      GST_OBJECT_LOCK (self);
-      priv->drm_device = g_value_dup_string (value);
-      GST_OBJECT_UNLOCK (self);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -462,17 +430,6 @@ scrollable_window_adjustment_changed_cb (GtkAdjustment * adjustment,
   return FALSE;
 }
 
-static void
-wl_window_map_cb (GstWlWindow * wl_window, GstGtkWaylandSink * self)
-{
-  GstGtkWaylandSinkPrivate *priv =
-      gst_gtk_wayland_sink_get_instance_private (self);
-
-  GST_DEBUG_OBJECT (self, "waylandsink surface is ready");
-
-  gtk_gst_base_widget_queue_draw (GTK_GST_BASE_WIDGET (priv->gtk_widget));
-}
-
 static void
 setup_wl_window (GstGtkWaylandSink * self)
 {
@@ -499,8 +456,6 @@ setup_wl_window (GstGtkWaylandSink * self)
         wl_surface, &priv->render_lock);
     gst_wl_window_set_rotate_method (priv->wl_window,
         priv->current_rotate_method);
-    g_signal_connect_object (priv->wl_window, "map",
-        G_CALLBACK (wl_window_map_cb), self, 0);
   }
 
   /* In order to position the subsurface correctly within a scrollable widget,
@@ -666,7 +621,6 @@ gst_gtk_wayland_sink_stop_on_main (GstGtkWaylandSink * self)
       g_signal_handler_disconnect (priv->gtk_window,
           priv->gtk_window_destroy_id);
     priv->gtk_window_destroy_id = 0;
-    g_clear_object (&priv->wl_window);
     gtk_widget_destroy (priv->gtk_window);
     priv->gtk_window = NULL;
   }
@@ -754,8 +708,6 @@ gst_gtk_wayland_sink_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_NULL:
-      g_clear_object (&priv->pool);
-      /* fallthrough */
     case GST_STATE_CHANGE_NULL_TO_NULL:
       gst_gtk_invoke_on_main ((GThreadFunc)
           gst_gtk_wayland_sink_stop_on_main, element);
@@ -766,6 +718,14 @@ gst_gtk_wayland_sink_change_state (GstElement * element,
         /* remove buffer from surface, show nothing */
         gst_wl_window_render (priv->wl_window, NULL, NULL);
       }
+
+      g_mutex_lock (&priv->render_lock);
+      if (priv->callback) {
+        wl_callback_destroy (priv->callback);
+        priv->callback = NULL;
+      }
+      priv->redraw_pending = FALSE;
+      g_mutex_unlock (&priv->render_lock);
       break;
     default:
       break;
@@ -788,21 +748,45 @@ gst_gtk_wayland_sink_get_caps (GstBaseSink * bsink, GstCaps * filter)
   g_mutex_lock (&priv->display_lock);
 
   if (priv->display) {
-    GValue format_list = G_VALUE_INIT;
+    GValue shm_list = G_VALUE_INIT, dmabuf_list = G_VALUE_INIT;
+    GValue value = G_VALUE_INIT;
+    GArray *formats;
+    gint i;
+    guint fmt;
+    GstVideoFormat gfmt;
 
-    g_value_init (&format_list, GST_TYPE_LIST);
-
-    /* Add corresponding dmabuf formats */
-    gst_wl_display_fill_dmabuf_format_list (priv->display, &format_list);
-    gst_structure_take_value (gst_caps_get_structure (caps, 0), "drm-format",
-        &format_list);
-
-    g_value_init (&format_list, GST_TYPE_LIST);
+    g_value_init (&shm_list, GST_TYPE_LIST);
+    g_value_init (&dmabuf_list, GST_TYPE_LIST);
 
     /* Add corresponding shm formats */
-    gst_wl_display_fill_shm_format_list (priv->display, &format_list);
+    formats = gst_wl_display_get_shm_formats (priv->display);
+    for (i = 0; i < formats->len; i++) {
+      fmt = g_array_index (formats, uint32_t, i);
+      gfmt = gst_wl_shm_format_to_video_format (fmt);
+      if (gfmt != GST_VIDEO_FORMAT_UNKNOWN) {
+        g_value_init (&value, G_TYPE_STRING);
+        g_value_set_static_string (&value, gst_video_format_to_string (gfmt));
+        gst_value_list_append_and_take_value (&shm_list, &value);
+      }
+    }
+
+    gst_structure_take_value (gst_caps_get_structure (caps, 0), "format",
+        &shm_list);
+
+    /* Add corresponding dmabuf formats */
+    formats = gst_wl_display_get_dmabuf_formats (priv->display);
+    for (i = 0; i < formats->len; i++) {
+      fmt = g_array_index (formats, uint32_t, i);
+      gfmt = gst_wl_dmabuf_format_to_video_format (fmt);
+      if (gfmt != GST_VIDEO_FORMAT_UNKNOWN) {
+        g_value_init (&value, G_TYPE_STRING);
+        g_value_set_static_string (&value, gst_video_format_to_string (gfmt));
+        gst_value_list_append_and_take_value (&dmabuf_list, &value);
+      }
+    }
+
     gst_structure_take_value (gst_caps_get_structure (caps, 1), "format",
-        &format_list);
+        &dmabuf_list);
 
     GST_DEBUG_OBJECT (self, "display caps: %" GST_PTR_FORMAT, caps);
   }
@@ -821,93 +805,30 @@ gst_gtk_wayland_sink_get_caps (GstBaseSink * bsink, GstCaps * filter)
   return caps;
 }
 
-static gboolean
-gst_gtk_wayland_update_pool (GstGtkWaylandSink * self, GstAllocator * allocator)
-{
-  GstGtkWaylandSinkPrivate *priv =
-      gst_gtk_wayland_sink_get_instance_private (self);
-  gsize size = priv->render_info.size;
-  GstStructure *config;
-
-  /* Pools with outstanding buffer cannot be reconfigured, so we must use
-   * a new pool. */
-  if (priv->pool) {
-    gst_buffer_pool_set_active (priv->pool, FALSE);
-    gst_object_unref (priv->pool);
-  }
-  priv->pool = gst_wl_video_buffer_pool_new ();
-  gst_object_ref_sink (priv->pool);
-
-  config = gst_buffer_pool_get_config (priv->pool);
-  gst_buffer_pool_config_set_params (config, priv->caps, size, 2, 0);
-  gst_buffer_pool_config_set_allocator (config, allocator, NULL);
-
-  if (!gst_buffer_pool_set_config (priv->pool, config))
-    return FALSE;
-
-  return gst_buffer_pool_set_active (priv->pool, TRUE);
-}
-
-static gboolean
-gst_gtk_wayland_activate_shm_pool (GstGtkWaylandSink * self)
-{
-  GstGtkWaylandSinkPrivate *priv =
-      gst_gtk_wayland_sink_get_instance_private (self);
-  GstAllocator *alloc = NULL;
-
-  if (priv->pool && gst_buffer_pool_is_active (priv->pool)) {
-    GstStructure *config = gst_buffer_pool_get_config (priv->pool);
-    gboolean is_shm = FALSE;
-
-    if (gst_buffer_pool_config_get_allocator (config, &alloc, NULL) && alloc)
-      is_shm = GST_IS_SHM_ALLOCATOR (alloc);
-
-    gst_structure_free (config);
-
-    if (is_shm)
-      return TRUE;
-  }
-
-  alloc = gst_shm_allocator_get ();
-  gst_gtk_wayland_update_pool (self, alloc);
-  gst_object_unref (alloc);
-
-  return TRUE;
-}
-
-static gboolean
-gst_gtk_wayland_activate_drm_dumb_pool (GstGtkWaylandSink * self)
+static GstBufferPool *
+gst_gtk_wayland_create_pool (GstGtkWaylandSink * self, GstCaps * caps)
 {
   GstGtkWaylandSinkPrivate *priv =
       gst_gtk_wayland_sink_get_instance_private (self);
+  GstBufferPool *pool = NULL;
+  GstStructure *structure;
+  gsize size = priv->video_info.size;
   GstAllocator *alloc;
 
-  if (!priv->drm_device)
-    return FALSE;
+  pool = gst_wl_video_buffer_pool_new ();
 
-  if (priv->pool && gst_buffer_pool_is_active (priv->pool)) {
-    GstStructure *config = gst_buffer_pool_get_config (priv->pool);
-    gboolean ret = FALSE;
-    gboolean is_drm_dumb = FALSE;
+  structure = gst_buffer_pool_get_config (pool);
+  gst_buffer_pool_config_set_params (structure, caps, size, 2, 0);
 
-    ret = gst_buffer_pool_config_get_allocator (config, &alloc, NULL);
-    gst_structure_free (config);
-
-    if (ret && alloc)
-      is_drm_dumb = GST_IS_DRM_DUMB_ALLOCATOR (alloc);
-
-    if (is_drm_dumb)
-      return TRUE;
+  alloc = gst_wl_shm_allocator_get ();
+  gst_buffer_pool_config_set_allocator (structure, alloc, NULL);
+  if (!gst_buffer_pool_set_config (pool, structure)) {
+    g_object_unref (pool);
+    pool = NULL;
   }
+  g_object_unref (alloc);
 
-  alloc = gst_drm_dumb_allocator_new_with_device_path (priv->drm_device);
-  if (!alloc)
-    return FALSE;
-
-  gst_gtk_wayland_update_pool (self, alloc);
-  gst_object_unref (alloc);
-
-  return TRUE;
+  return pool;
 }
 
 static gboolean
@@ -917,53 +838,30 @@ gst_gtk_wayland_sink_set_caps (GstBaseSink * bsink, GstCaps * caps)
   GstGtkWaylandSinkPrivate *priv =
       gst_gtk_wayland_sink_get_instance_private (self);
   gboolean use_dmabuf;
+  GstVideoFormat format;
 
   GST_DEBUG_OBJECT (self, "set caps %" GST_PTR_FORMAT, caps);
 
-  if (gst_video_is_dma_drm_caps (caps)) {
-    if (!gst_video_info_dma_drm_from_caps (&priv->drm_info, caps))
-      goto invalid_format;
+  /* extract info from caps */
+  if (!gst_video_info_from_caps (&priv->video_info, caps))
+    goto invalid_format;
 
-    if (!gst_video_info_dma_drm_to_video_info (&priv->drm_info,
-            &priv->render_info))
-      goto invalid_format;
-  } else {
-    /* extract info from caps */
-    if (!gst_video_info_from_caps (&priv->render_info, caps))
-      goto invalid_format;
+  format = GST_VIDEO_INFO_FORMAT (&priv->video_info);
+  priv->video_info_changed = TRUE;
 
-    if (!gst_video_info_dma_drm_from_video_info (&priv->drm_info,
-            &priv->render_info, DRM_FORMAT_MOD_LINEAR))
-      gst_video_info_dma_drm_init (&priv->drm_info);
-  }
-
-  priv->have_mastering_info =
-      gst_video_mastering_display_info_from_caps (&priv->minfo, caps);
-  priv->have_light_info =
-      gst_video_content_light_level_from_caps (&priv->linfo, caps);
-
-  priv->render_info_changed = TRUE;
-  priv->skip_dumb_buffer_copy = FALSE;
-
-  /* free pooled buffer used with previous caps */
-  if (priv->pool) {
-    gst_buffer_pool_set_active (priv->pool, FALSE);
-    gst_clear_object (&priv->pool);
-  }
+  /* create a new pool for the new caps */
+  if (priv->pool)
+    gst_object_unref (priv->pool);
+  priv->pool = gst_gtk_wayland_create_pool (self, caps);
 
   use_dmabuf = gst_caps_features_contains (gst_caps_get_features (caps, 0),
       GST_CAPS_FEATURE_MEMORY_DMABUF);
 
   /* validate the format base on the memory type. */
   if (use_dmabuf) {
-    if (!gst_wl_display_check_format_for_dmabuf (priv->display,
-            &priv->drm_info))
-      goto unsupported_drm_format;
-  } else if (!gst_wl_display_check_format_for_shm (priv->display,
-          &priv->render_info)) {
-    /* Note: we still support dmabuf in this case, but formats must also be
-     * supported on SHM interface to ensure a fallback is possible as we are
-     * not guarantied we'll get dmabuf in the buffers. */
+    if (!gst_wl_display_check_format_for_dmabuf (priv->display, format))
+      goto unsupported_format;
+  } else if (!gst_wl_display_check_format_for_shm (priv->display, format)) {
     goto unsupported_format;
   }
 
@@ -977,21 +875,13 @@ gst_gtk_wayland_sink_set_caps (GstBaseSink * bsink, GstCaps * caps)
   }
 
   if (!gtk_gst_base_widget_set_format (GTK_GST_BASE_WIDGET (priv->gtk_widget),
-          &priv->render_info)) {
+          &priv->video_info)) {
     GST_OBJECT_UNLOCK (self);
     return FALSE;
   }
-
-  /* Ensure queue_draw get executed and internal display size get initialized.
-   * This does not happen otherwise as we don't draw in the widget
-   */
-  gtk_gst_base_widget_queue_draw (GTK_GST_BASE_WIDGET (priv->gtk_widget));
-
   GST_OBJECT_UNLOCK (self);
 
-  /* Will be used to create buffer pools */
-  gst_caps_replace (&priv->caps, caps);
-  priv->video_info = priv->render_info;
+  priv->use_dmabuf = use_dmabuf;
 
   return TRUE;
 
@@ -1001,18 +891,10 @@ invalid_format:
         "Could not locate image format from caps %" GST_PTR_FORMAT, caps);
     return FALSE;
   }
-unsupported_drm_format:
-  {
-    GST_ERROR_OBJECT (self, "DRM format %" GST_FOURCC_FORMAT
-        " is not available on the display",
-        GST_FOURCC_ARGS (priv->drm_info.drm_fourcc));
-    return FALSE;
-  }
 unsupported_format:
   {
     GST_ERROR_OBJECT (self, "Format %s is not available on the display",
-        gst_video_format_to_string (GST_VIDEO_INFO_FORMAT
-            (&priv->render_info)));
+        gst_video_format_to_string (format));
     return FALSE;
   }
 }
@@ -1023,145 +905,75 @@ gst_gtk_wayland_sink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
   GstGtkWaylandSink *self = GST_GTK_WAYLAND_SINK (bsink);
   GstGtkWaylandSinkPrivate *priv =
       gst_gtk_wayland_sink_get_instance_private (self);
-  GstAllocator *allocator = NULL;
   GstCaps *caps;
   GstBufferPool *pool = NULL;
   gboolean need_pool;
-  guint size;
+  GstAllocator *alloc;
 
   gst_query_parse_allocation (query, &caps, &need_pool);
 
-  if (caps == NULL)
-    return FALSE;
+  if (need_pool)
+    pool = gst_gtk_wayland_create_pool (self, caps);
 
-  if (gst_video_is_dma_drm_caps (caps)) {
-    GstVideoInfoDmaDrm drm_info;
-
-    if (!gst_video_info_dma_drm_from_caps (&drm_info, caps))
-      return FALSE;
-
-    size = drm_info.vinfo.size;
-  } else {
-    GstVideoInfo vinfo;
-
-    /* extract info from caps */
-    if (!gst_video_info_from_caps (&vinfo, caps))
-      return FALSE;
-
-    size = vinfo.size;
-
-    allocator = gst_udmabuf_allocator_get ();
-    if (!allocator)
-      allocator = gst_shm_allocator_get ();
-  }
-
-  if (need_pool && !gst_video_is_dma_drm_caps (caps)) {
-    GstStructure *config;
-
-    if (GST_IS_UDMABUF_ALLOCATOR (allocator)) {
-      pool = gst_video_dmabuf_pool_new ();
-    } else {
-      pool = gst_wl_video_buffer_pool_new ();
-      config = gst_buffer_pool_get_config (pool);
-      gst_buffer_pool_config_set_params (config, caps, size, 2, 0);
-      gst_buffer_pool_config_set_allocator (config,
-          gst_object_ref (allocator), NULL);
-      gst_buffer_pool_set_config (pool, config);
-    }
-  }
-
-  gst_query_add_allocation_pool (query, pool, size, 2, 0);
+  gst_query_add_allocation_pool (query, pool, priv->video_info.size, 2, 0);
   if (pool)
     g_object_unref (pool);
 
-  if (!gst_video_is_dma_drm_caps (caps))
-    gst_query_add_allocation_param (query, allocator, NULL);
-
+  alloc = gst_wl_shm_allocator_get ();
+  gst_query_add_allocation_param (query, alloc, NULL);
   gst_query_add_allocation_meta (query, GST_VIDEO_META_API_TYPE, NULL);
+  g_object_unref (alloc);
 
-  if (gst_wl_display_get_viewporter (priv->display))
-    gst_query_add_allocation_meta (query, GST_VIDEO_CROP_META_API_TYPE, NULL);
-
-  gst_clear_object (&allocator);
   return TRUE;
 }
 
+static void
+frame_redraw_callback (void *data, struct wl_callback *callback, uint32_t time)
+{
+  GstGtkWaylandSink *self = data;
+  GstGtkWaylandSinkPrivate *priv =
+      gst_gtk_wayland_sink_get_instance_private (self);
+
+  GST_LOG_OBJECT (self, "frame_redraw_cb");
+
+  g_mutex_lock (&priv->render_lock);
+  priv->redraw_pending = FALSE;
+
+  if (priv->callback) {
+    wl_callback_destroy (callback);
+    priv->callback = NULL;
+  }
+  g_mutex_unlock (&priv->render_lock);
+}
+
+static const struct wl_callback_listener frame_callback_listener = {
+  frame_redraw_callback
+};
+
 /* must be called with the render lock */
-static gboolean
+static void
 render_last_buffer (GstGtkWaylandSink * self, gboolean redraw)
 {
   GstGtkWaylandSinkPrivate *priv =
       gst_gtk_wayland_sink_get_instance_private (self);
   GstWlBuffer *wlbuffer;
   const GstVideoInfo *info = NULL;
-  const GstVideoMasteringDisplayInfo *minfo = NULL;
-  const GstVideoContentLightLevel *linfo = NULL;
-
-  if (!priv->wl_window)
-    return FALSE;
+  struct wl_surface *surface;
+  struct wl_callback *callback;
 
   wlbuffer = gst_buffer_get_wl_buffer (priv->display, priv->last_buffer);
+  surface = gst_wl_window_get_wl_surface (priv->wl_window);
 
-  if (G_UNLIKELY (priv->render_info_changed && !redraw)) {
-    info = &priv->render_info;
+  priv->redraw_pending = TRUE;
+  callback = wl_surface_frame (surface);
+  priv->callback = callback;
+  wl_callback_add_listener (callback, &frame_callback_listener, self);
 
-    if (priv->have_mastering_info)
-      minfo = &priv->minfo;
-
-    if (priv->have_light_info)
-      linfo = &priv->linfo;
-
-    priv->render_info_changed = FALSE;
-  }
-  return gst_wl_window_render_hdr (priv->wl_window, wlbuffer, info, minfo,
-      linfo);
-}
-
-static GstFlowReturn
-gst_gtk_wayland_sink_copy_frame (GstGtkWaylandSink * self,
-    GstBuffer * src_buffer, GstBuffer * dst_buffer)
-{
-  GstGtkWaylandSinkPrivate *priv =
-      gst_gtk_wayland_sink_get_instance_private (self);
-  GstVideoFrame src, dst;
-
-  if (!gst_video_frame_map (&dst, &priv->video_info, dst_buffer, GST_MAP_WRITE))
-    goto dst_map_failed;
-
-  if (!gst_video_frame_map (&src, &priv->video_info, src_buffer, GST_MAP_READ)) {
-    gst_video_frame_unmap (&dst);
-    goto src_map_failed;
-  }
-
-  gst_video_frame_copy (&dst, &src);
-
-  gst_video_frame_unmap (&src);
-  gst_video_frame_unmap (&dst);
-
-  /* Also copy the crop meta so its offloaded */
-  GstVideoCropMeta *src_cmeta = gst_buffer_get_video_crop_meta (src_buffer);
-  if (src_cmeta) {
-    GstVideoCropMeta *dst_cmeta = gst_buffer_add_video_crop_meta (dst_buffer);
-    dst_cmeta->x = src_cmeta->x;
-    dst_cmeta->y = src_cmeta->y;
-    dst_cmeta->width = src_cmeta->width;
-    dst_cmeta->height = src_cmeta->height;
-  }
-
-  return GST_FLOW_OK;
-
-src_map_failed:
-  {
-    GST_ELEMENT_ERROR (self, RESOURCE, READ,
-        ("Video memory can not be read from userspace."), (NULL));
-    return GST_FLOW_ERROR;
-  }
-dst_map_failed:
-  {
-    GST_ELEMENT_ERROR (self, RESOURCE, WRITE,
-        ("Video memory can not be written from userspace."), (NULL));
-    return GST_FLOW_ERROR;
+  if (G_UNLIKELY (priv->video_info_changed && !redraw)) {
+    info = &priv->video_info;
+    priv->video_info_changed = FALSE;
   }
+  gst_wl_window_render (priv->wl_window, wlbuffer, info);
 }
 
 static GstFlowReturn
@@ -1172,6 +984,9 @@ gst_gtk_wayland_sink_show_frame (GstVideoSink * vsink, GstBuffer * buffer)
       gst_gtk_wayland_sink_get_instance_private (self);
   GstBuffer *to_render;
   GstWlBuffer *wlbuffer;
+  GstVideoMeta *vmeta;
+  GstVideoFormat format;
+  GstVideoInfo old_vinfo;
   GstMemory *mem;
   struct wl_buffer *wbuf = NULL;
 
@@ -1184,37 +999,14 @@ gst_gtk_wayland_sink_show_frame (GstVideoSink * vsink, GstBuffer * buffer)
   if (!priv->wl_window) {
     GST_LOG_OBJECT (self,
         "buffer %" GST_PTR_FORMAT " dropped (waiting for window)", buffer);
-    ret = GST_BASE_SINK_FLOW_DROPPED;
     goto done;
   }
 
-  /*
-   * The GstVideoFrame fast copy can't crop, make sure the internal pool
-   * allocated buffers large enough to hold the padded frames.
-   */
-  if (gst_buffer_get_video_crop_meta (buffer)) {
-    gint padded_width, padded_height;
-    GstVideoMeta *vmeta;
-    GstStructure *s;
-
-    vmeta = gst_buffer_get_video_meta (buffer);
-    priv->caps = gst_caps_make_writable (priv->caps);
-    s = gst_caps_get_structure (priv->caps, 0);
-    gst_structure_get (s, "width", G_TYPE_INT, &padded_width,
-        "height", G_TYPE_INT, &padded_height, NULL);
-
-    if (vmeta->width != padded_width || vmeta->height != padded_height) {
-      gst_structure_set (s, "width", G_TYPE_INT, vmeta->width,
-          "height", G_TYPE_INT, vmeta->height, NULL);
-
-      if (priv->pool) {
-        gst_buffer_pool_set_active (priv->pool, FALSE);
-        gst_clear_object (&priv->pool);
-      }
-
-      gst_video_info_set_format (&priv->video_info, vmeta->format,
-          vmeta->width, vmeta->height);
-    }
+  /* drop buffers until we get a frame callback */
+  if (priv->redraw_pending) {
+    GST_LOG_OBJECT (self, "buffer %" GST_PTR_FORMAT " dropped (redraw pending)",
+        buffer);
+    goto done;
   }
 
   /* make sure that the application has called set_render_rectangle() */
@@ -1235,11 +1027,24 @@ gst_gtk_wayland_sink_show_frame (GstVideoSink * vsink, GstBuffer * buffer)
   /* update video info from video meta */
   mem = gst_buffer_peek_memory (buffer, 0);
 
+  old_vinfo = priv->video_info;
+  vmeta = gst_buffer_get_video_meta (buffer);
+  if (vmeta) {
+    gint i;
+
+    for (i = 0; i < vmeta->n_planes; i++) {
+      priv->video_info.offset[i] = vmeta->offset[i];
+      priv->video_info.stride[i] = vmeta->stride[i];
+    }
+    priv->video_info.size = gst_buffer_get_size (buffer);
+  }
+
   GST_LOG_OBJECT (self,
       "buffer %" GST_PTR_FORMAT " does not have a wl_buffer from our "
       "display, creating it", buffer);
 
-  if (gst_wl_display_check_format_for_dmabuf (priv->display, &priv->drm_info)) {
+  format = GST_VIDEO_INFO_FORMAT (&priv->video_info);
+  if (gst_wl_display_check_format_for_dmabuf (priv->display, format)) {
     guint i, nb_dmabuf = 0;
 
     for (i = 0; i < gst_buffer_n_memory (buffer); i++)
@@ -1248,58 +1053,22 @@ gst_gtk_wayland_sink_show_frame (GstVideoSink * vsink, GstBuffer * buffer)
 
     if (nb_dmabuf && (nb_dmabuf == gst_buffer_n_memory (buffer)))
       wbuf = gst_wl_linux_dmabuf_construct_wl_buffer (buffer, priv->display,
-          &priv->drm_info);
-
-    if (!wbuf && !priv->skip_dumb_buffer_copy) {
-      /* DMABuf did not work, let try and make this a dmabuf, it does not matter
-       * if it was a SHM since the compositor needs to copy that anyway, and
-       * offloading the compositor from a copy helps maintaining a smoother
-       * desktop.
-       */
-
-      if (!gst_gtk_wayland_activate_drm_dumb_pool (self)) {
-        priv->skip_dumb_buffer_copy = TRUE;
-        goto handle_shm;
-      }
-
-      ret = gst_buffer_pool_acquire_buffer (priv->pool, &to_render, NULL);
-      if (ret != GST_FLOW_OK)
-        goto no_buffer;
-
-      wlbuffer = gst_buffer_get_wl_buffer (priv->display, to_render);
-
-      /* attach a wl_buffer if there isn't one yet */
-      if (G_UNLIKELY (!wlbuffer)) {
-        wbuf = gst_wl_linux_dmabuf_construct_wl_buffer (to_render,
-            priv->display, &priv->drm_info);
-
-        if (G_UNLIKELY (!wbuf)) {
-          GST_WARNING_OBJECT (self, "failed to import DRM Dumb dmabuf");
-          gst_clear_buffer (&to_render);
-          priv->skip_dumb_buffer_copy = TRUE;
-          goto handle_shm;
-        }
-
-        wlbuffer = gst_buffer_add_wl_buffer (to_render, wbuf, priv->display);
-      }
-
-      ret = gst_gtk_wayland_sink_copy_frame (self, buffer, to_render);
-      if (ret != GST_FLOW_OK)
-        goto done;
-
-      goto render;
-    }
+          &priv->video_info);
   }
 
-handle_shm:
-  if (!wbuf && gst_wl_display_check_format_for_shm (priv->display,
-          &priv->render_info)) {
+  if (!wbuf && gst_wl_display_check_format_for_shm (priv->display, format)) {
     if (gst_buffer_n_memory (buffer) == 1 && gst_is_fd_memory (mem))
       wbuf = gst_wl_shm_memory_construct_wl_buffer (mem, priv->display,
-          &priv->render_info);
+          &priv->video_info);
 
     /* If nothing worked, copy into our internal pool */
     if (!wbuf) {
+      GstVideoFrame src, dst;
+      GstVideoInfo src_info = priv->video_info;
+
+      /* rollback video info changes */
+      priv->video_info = old_vinfo;
+
       /* we don't know how to create a wl_buffer directly from the provided
        * memory, so we have to copy the data to shm memory that we know how
        * to handle... */
@@ -1308,10 +1077,25 @@ handle_shm:
           "buffer %" GST_PTR_FORMAT " cannot have a wl_buffer, "
           "copying to wl_shm memory", buffer);
 
+      /* priv->pool always exists (created in set_caps), but it may not
+       * be active if upstream is not using it */
+      if (!gst_buffer_pool_is_active (priv->pool)) {
+        GstStructure *config;
+        GstCaps *caps;
 
-      /* ensure the internal pool is configured for SHM */
-      if (!gst_gtk_wayland_activate_shm_pool (self))
-        goto activate_failed;
+        config = gst_buffer_pool_get_config (priv->pool);
+        gst_buffer_pool_config_get_params (config, &caps, NULL, NULL, NULL);
+
+        /* revert back to default strides and offsets */
+        gst_video_info_from_caps (&priv->video_info, caps);
+        gst_buffer_pool_config_set_params (config, caps, priv->video_info.size,
+            2, 0);
+
+        /* This is a video pool, it should not fail with basic settings */
+        if (!gst_buffer_pool_set_config (priv->pool, config) ||
+            !gst_buffer_pool_set_active (priv->pool, TRUE))
+          goto activate_failed;
+      }
 
       ret = gst_buffer_pool_acquire_buffer (priv->pool, &to_render, NULL);
       if (ret != GST_FLOW_OK)
@@ -1323,7 +1107,7 @@ handle_shm:
       if (G_UNLIKELY (!wlbuffer)) {
         mem = gst_buffer_peek_memory (to_render, 0);
         wbuf = gst_wl_shm_memory_construct_wl_buffer (mem, priv->display,
-            &priv->render_info);
+            &priv->video_info);
 
         if (G_UNLIKELY (!wbuf))
           goto no_wl_buffer_shm;
@@ -1331,9 +1115,19 @@ handle_shm:
         wlbuffer = gst_buffer_add_wl_buffer (to_render, wbuf, priv->display);
       }
 
-      ret = gst_gtk_wayland_sink_copy_frame (self, buffer, to_render);
-      if (ret != GST_FLOW_OK)
-        goto done;
+      if (!gst_video_frame_map (&dst, &priv->video_info, to_render,
+              GST_MAP_WRITE))
+        goto dst_map_failed;
+
+      if (!gst_video_frame_map (&src, &src_info, buffer, GST_MAP_READ)) {
+        gst_video_frame_unmap (&dst);
+        goto src_map_failed;
+      }
+
+      gst_video_frame_copy (&dst, &src);
+
+      gst_video_frame_unmap (&src);
+      gst_video_frame_unmap (&dst);
 
       goto render;
     }
@@ -1354,8 +1148,7 @@ render:
   }
 
   gst_buffer_replace (&priv->last_buffer, to_render);
-  if (!render_last_buffer (self, FALSE))
-    ret = GST_BASE_SINK_FLOW_DROPPED;
+  render_last_buffer (self, FALSE);
 
   if (buffer != to_render)
     gst_buffer_unref (to_render);
@@ -1371,29 +1164,39 @@ no_window_size:
   }
 no_buffer:
   {
-    GST_ELEMENT_ERROR (self, RESOURCE, FAILED,
-        ("could not create buffer"), (NULL));
-    ret = GST_FLOW_ERROR;
+    GST_WARNING_OBJECT (self, "could not create buffer");
     goto done;
   }
 no_wl_buffer_shm:
   {
-    GST_ELEMENT_ERROR (self, RESOURCE, FAILED,
-        ("could not create wl_buffer out of wl_shm memory"), (NULL));
+    GST_ERROR_OBJECT (self, "could not create wl_buffer out of wl_shm memory");
     ret = GST_FLOW_ERROR;
     goto done;
   }
 no_wl_buffer:
   {
-    GST_ELEMENT_ERROR (self, RESOURCE, FAILED,
-        ("buffer %" GST_PTR_FORMAT " cannot have a wl_buffer", buffer), (NULL));
+    GST_ERROR_OBJECT (self,
+        "buffer %" GST_PTR_FORMAT " cannot have a wl_buffer", buffer);
     ret = GST_FLOW_ERROR;
     goto done;
   }
 activate_failed:
   {
-    GST_ELEMENT_ERROR (self, RESOURCE, FAILED,
-        ("failed to activate bufferpool."), (NULL));
+    GST_ERROR_OBJECT (self, "failed to activate bufferpool.");
+    ret = GST_FLOW_ERROR;
+    goto done;
+  }
+src_map_failed:
+  {
+    GST_ELEMENT_ERROR (self, RESOURCE, READ,
+        ("Video memory can not be read from userspace."), (NULL));
+    ret = GST_FLOW_ERROR;
+    goto done;
+  }
+dst_map_failed:
+  {
+    GST_ELEMENT_ERROR (self, RESOURCE, WRITE,
+        ("Video memory can not be written from userspace."), (NULL));
     ret = GST_FLOW_ERROR;
     goto done;
   }
