@@ -3,7 +3,7 @@
  * Copyright (C) 2005 Thomas Vander Stichele <thomas@apestaart.org>
  * Copyright (C) 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
  * Copyright (C) 2008 Michael Sheldon <mike@mikeasoft.com>
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -46,9 +46,7 @@
 #ifndef __GST_SKIN_DETECT_H__
 #define __GST_SKIN_DETECT_H__
 
-#include <gst/gst.h>
-
-#include <gstopencvvideofilter.h>
+#include <gst/opencv/gstopencvvideofilter.h>
 
 G_BEGIN_DECLS
 /* #defines don't like whitespacey bits */
@@ -73,10 +71,10 @@ struct _GstSkinDetect
   gint method;
   gint width, height;
 
-  IplImage *cvChA, *cvRGB, *cvSkin;
-  IplImage *cvHSV, *cvH, *cvH2, *cvS, *cvV, *cvSkinPixels1;
-  IplImage *cvR, *cvG, *cvB, *cvAll, *cvR2, *cvRp, *cvGp, *cvRp2, *cvGp2,
-      *cvdraft, *cvSkinPixels2;
+  cv::Mat cvChA, cvRGB;
+  cv::Mat cvHSV, cvH, cvH2, cvS, cvV, cvSkinPixels1;
+  cv::Mat cvR, cvG, cvB, cvAll, cvR2, cvRp, cvGp, cvRp2, cvGp2,
+      cvdraft, cvSkinPixels2;
 };
 
 struct _GstSkinDetectClass
@@ -86,7 +84,7 @@ struct _GstSkinDetectClass
 
 GType gst_skin_detect_get_type (void);
 
-gboolean gst_skin_detect_plugin_init (GstPlugin * plugin);
+GST_ELEMENT_REGISTER_DECLARE (skindetect);
 
 G_END_DECLS
 #endif /* __GST_SKIN_DETECT_H__ */
