@@ -79,13 +79,7 @@ struct _GstVulkanBufferPoolClass
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstVulkanBufferPool, gst_object_unref);
 
 GST_VULKAN_API
-GstBufferPool *gst_vulkan_buffer_pool_new               (GstVulkanDevice * device) G_GNUC_WARN_UNUSED_RESULT;
-
-GST_VULKAN_API
-void            gst_vulkan_buffer_pool_config_set_allocation_params
-                                                        (GstStructure * config,
-                                                         VkBufferUsageFlags usage,
-                                                         VkMemoryPropertyFlags mem_properties);
+GstBufferPool *gst_vulkan_buffer_pool_new (GstVulkanDevice * device);
 
 G_END_DECLS
 
