@@ -366,8 +366,6 @@ namespace amf
         }
 
 #if (__cplusplus == 201103L) || defined(__GXX_EXPERIMENTAL_CXX0X) || (_MSC_VER >= 1600)
-#pragma warning (push)
-#pragma warning (disable : 26439) //This kind of function may not throw. Declare it 'noexcept'.
         String(String&& p_other) : m_Str(NULL)
         {
             operator=(p_other);
@@ -500,7 +498,6 @@ namespace amf
             p_other.m_Str = NULL;    //    Transfer the ownership
             return *this;
         }
-#pragma warning (pop)
 #endif
         wchar_t& operator[](size_t index)
         {
@@ -758,7 +755,7 @@ namespace amf
             switch(AMFVariantGetType(pSrc))
             {
             case AMF_VARIANT_EMPTY:
-                errRet = AMFVariantInit(pDest);
+                errRet = AMFVariantClear(pDest);
                 break;
             case AMF_VARIANT_BOOL:
                 errRet = AMFVariantAssignBool(pDest, AMFVariantBool(pSrc));
@@ -1204,21 +1201,21 @@ namespace amf
     {
         res = AMF_OK;
         char buff[0xFF];
-        sprintf(buff, "%u,%u", value.num, value.den);
+        sprintf(buff, "%d,%d", value.num, value.den);
         return buff;
     }
     static AMF_INLINE AMFVariant::String AMF_STD_CALL AMFConvertRatioToString(const AMFRatio& value, AMF_RESULT& res)
     {
         res = AMF_OK;
         char buff[0xFF];
-        sprintf(buff, "%u,%u", value.num, value.den);
+        sprintf(buff, "%d,%d", value.num, value.den);
         return buff;
     }
     static AMF_INLINE AMFVariant::String AMF_STD_CALL AMFConvertColorToString(const AMFColor& value, AMF_RESULT& res)
     {
         res = AMF_OK;
         char buff[0xFF];
-        sprintf(buff, "%u,%u,%u,%u", value.r, value.g, value.b, value.a);
+        sprintf(buff, "%d,%d,%d,%d", value.r, value.g, value.b, value.a);
         return buff;
     }
 
@@ -1349,7 +1346,7 @@ namespace amf
         int readElements = 0;
         if(value.size() > 0)
         {
-            readElements = sscanf(value.c_str(), "%u,%u", &tmp.num, &tmp.den);
+            readElements = sscanf(value.c_str(), "%d,%d", &tmp.num, &tmp.den);
         }
         if(readElements)
         {
@@ -1365,7 +1362,7 @@ namespace amf
         int readElements = 0;
         if(value.size() > 0)
         {
-            readElements = sscanf(value.c_str(), "%u,%u", &tmp.num, &tmp.den);
+            readElements = sscanf(value.c_str(), "%d,%d", &tmp.num, &tmp.den);
         }
         if(readElements)
         {
@@ -1584,7 +1581,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_BOOL;
@@ -1598,7 +1595,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_INT64;
@@ -1612,7 +1609,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_DOUBLE;
@@ -1626,7 +1623,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if (errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_FLOAT;
@@ -1641,7 +1638,7 @@ namespace amf
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pValue);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             const size_t size = (strlen(pValue) + 1);
@@ -1665,7 +1662,7 @@ namespace amf
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pValue);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             const size_t size = (wcslen(pValue) + 1);
@@ -1689,7 +1686,7 @@ namespace amf
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
         //AMF_VARIANT_RETURN_IF_INVALID_POINTER(pValue);//can be NULL
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_INTERFACE;
@@ -1718,7 +1715,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_RECT;
@@ -1739,7 +1736,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_SIZE;
@@ -1776,7 +1773,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_POINT;
@@ -1790,7 +1787,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if (errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_FLOAT_SIZE;
@@ -1804,7 +1801,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if (errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_FLOAT_POINT2D;
@@ -1818,7 +1815,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if (errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_FLOAT_POINT3D;
@@ -1832,7 +1829,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if (errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_FLOAT_VECTOR4D;
@@ -1853,7 +1850,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_RATE;
@@ -1874,7 +1871,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_RATIO;
@@ -1895,7 +1892,7 @@ namespace amf
         AMF_RESULT errRet = AMF_OK;
         AMF_VARIANT_RETURN_IF_INVALID_POINTER(pDest);
 
-        errRet = AMFVariantInit(pDest);
+        errRet = AMFVariantClear(pDest);
         if(errRet == AMF_OK)
         {
             pDest->type = AMF_VARIANT_COLOR;
@@ -1983,7 +1980,6 @@ namespace amf
     //-------------------------------------------------------------------------------------------------
     AMF_INLINE AMFVariant& AMFVariant::operator=(const AMFVariantStruct& other)
     {
-        AMFVariantClear(this);
         AMFVariantCopy(this, const_cast<AMFVariantStruct*>(&other));
         return *this;
     }
@@ -1992,7 +1988,6 @@ namespace amf
     {
         if(pOther != NULL)
         {
-            AMFVariantClear(this);
             AMFVariantCopy(this, const_cast<AMFVariantStruct*>(pOther));
         }
         return *this;
@@ -2000,7 +1995,6 @@ namespace amf
     //-------------------------------------------------------------------------------------------------
     AMF_INLINE AMFVariant& AMFVariant::operator=(const AMFVariant& other)
     {
-        AMFVariantClear(this);
         AMFVariantCopy(this,
                 const_cast<AMFVariantStruct*>(static_cast<const AMFVariantStruct*>(&other)));
         return *this;
@@ -2009,7 +2003,6 @@ namespace amf
     template<typename T>
     AMFVariant& AMFVariant::operator=(const AMFInterfacePtr_T<T>& value)
     {
-        AMFVariantClear(this);
         AMFVariantAssignInterface(this, value);
         return *this;
     }
